from django.test import TestCase
from partitialajax.contrib import html
from partitialajax.exceptions import InvalidSelectorString
# Create your tests here.


class HTMLSelectorTestCase(TestCase):

    def test_id_split(self):
        info = html.split_selector("#testIDString")
        self.assertEqual(info["id"], "testIDString")
        self.assertEqual(info["element"], None)
        self.assertEqual(len(info["class_list"]), 0)
        self.assertEqual(len(info["attribute_list"]), 0)

    def test_double_id(self):
        with self.assertRaises(InvalidSelectorString) as context:
            html.split_selector("#testIDString#anotherID")

    def test_element_split(self):
        info = html.split_selector("div")
        self.assertEqual(info["id"], None)
        self.assertEqual(info["element"], "div")
        self.assertEqual(len(info["class_list"]), 0)
        self.assertEqual(len(info["attribute_list"]), 0)

    def test_class_split(self):
        info = html.split_selector(".testClassString")
        self.assertEqual(info["id"], None)
        self.assertEqual(info["element"], None)
        self.assertEqual(len(info["class_list"]), 1)
        self.assertEqual(list(info["class_list"])[0], "testClassString")
        self.assertEqual(len(info["attribute_list"]), 0)

    def test_class_list_split(self):
        info = html.split_selector(".testClassString.classA.classB")
        self.assertEqual(info["id"], None)
        self.assertEqual(info["element"], None)
        self.assertEqual(len(info["class_list"]), 3)
        self.assertEqual(info["class_list"], {"testClassString", "classA", "classB"})
        self.assertEqual(len(info["attribute_list"]), 0)


    def test_atttribute_split(self):
        info = html.split_selector("[attri=\"value\"]")
        self.assertEqual(info["id"], None)
        self.assertEqual(info["element"], None)
        self.assertEqual(len(info["class_list"]), 0)
        self.assertEqual(len(info["attribute_list"].items()), 1)
        self.assertEqual(info["attribute_list"], {"attri": "value"})

    def test_atttribute_list_split(self):
        info = html.split_selector('[attra="vala"][attrb="valb"][attrc="valc"][attrd="vald"]')
        self.assertEqual(info["id"], None)
        self.assertEqual(info["element"], None)
        self.assertEqual(len(info["class_list"]), 0)
        self.assertEqual(len(info["attribute_list"].items()), 4)
        self.assertEqual(info["attribute_list"], {"attra": "vala", "attrb": "valb", "attrc": "valc", "attrd": "vald"})

    def test_element_with_id(self):
        info = html.split_selector('superElement#elementID')
        self.assertEqual(info["element"], "superElement")
        self.assertEqual(info["id"], "elementID")
        self.assertEqual(len(info["class_list"]), 0)
        self.assertEqual(len(info["attribute_list"].items()), 0)

    def test_element_with_class(self):
        info = html.split_selector('superElement.elementClass')
        self.assertEqual(info["element"], "superElement")
        self.assertEqual(info["id"], None)
        self.assertEqual(len(info["class_list"]), 1)
        self.assertEqual(info["class_list"], {"elementClass"})
        self.assertEqual(len(info["attribute_list"].items()), 0)

    def test_element_with_class_and_id(self):
        info = html.split_selector('superElement.elementClass#elementID')
        self.assertEqual(info["element"], "superElement")
        self.assertEqual(info["id"], "elementID")
        self.assertEqual(len(info["class_list"]), 1)
        self.assertEqual(info["class_list"], {"elementClass"})
        self.assertEqual(len(info["attribute_list"].items()), 0)

        info = html.split_selector('superElement#elementID.elementClass')
        self.assertEqual(info["element"], "superElement")
        self.assertEqual(info["id"], "elementID")
        self.assertEqual(len(info["class_list"]), 1)
        self.assertEqual(info["class_list"], {"elementClass"})
        self.assertEqual(len(info["attribute_list"].items()), 0)

        info = html.split_selector('superElement#elementID.elementClass.anotherClass')
        self.assertEqual(info["element"], "superElement")
        self.assertEqual(info["id"], "elementID")
        self.assertEqual(len(info["class_list"]), 2)
        self.assertEqual(info["class_list"], {"elementClass", "anotherClass"})
        self.assertEqual(len(info["attribute_list"].items()), 0)

    def test_full_split(self):
        info = html.split_selector('superElement#singleElementID.elementClass[data-attribute="data-value"].anotherClass[data-anotherattr="foobar"]')
        self.assertEqual(info["element"], "superElement")
        self.assertEqual(info["id"], "singleElementID")
        self.assertEqual(len(info["class_list"]), 2)
        self.assertEqual(info["class_list"], {"elementClass", "anotherClass"})
        self.assertEqual(len(info["attribute_list"].items()), 2)
        self.assertEqual(info["attribute_list"], {"data-attribute": "data-value", "data-anotherattr":"foobar"})



from django.shortcuts import render
from django.http import JsonResponse
from django.template.loader import get_template
from django.template import Template
from django.views.generic import View
import requests


class PartitialAjaxMixin:
    """
        Add this Mixin to your View to activate the useage of PartitialAjax
    """

    #: Define here all partitial html files used in the template_name File
    #: Write as follows: {"#foobar": "myapp/partitial/foo.html"}
    partitial_list = dict()

    def get_partitial_list(self, *args, **kwargs):
        """
            Returns a dict with keys as selectors and values as partitial file paths
            :return: dict
            :Example:
            {"#foo": "myapp/partitials/foo.html"}
        """
        if len(self.partitial_list) <= 0:
            raise ValueError("partial list must not be empty")
        return self.partitial_list

    def get_partitial(self, origin, context):
        """
            Collect Partitial Contents from template or remote url

            :param origin: tuple or string; if tuple: the first value is the path to template or remote the second is the keyword "template", "remote"
            :param context: context used for rendering (remote paths also renderd with this context)
            :return:
        """
        if origin is tuple:
            if "template" in origin[1]:
                tpl = get_template(origin[0])
                return {
                    "template": tpl,
                    "content": tpl.render(context, request=self.request)
                }
            elif "remote" in origin[1]:
                with requests.get(origin[0]) as r:
                    tpl = Template(r.text)
                return {
                    "path": origin[0],
                    "content": tpl.render(context, request=self.request)
                }
            raise ValueError(f"Unknown origin {origin[1]}")
        else:
            tpl = get_template(origin)
            return {
                "template": tpl,
                "content": tpl.render(context, request=self.request)
            }

    def get_partitial_context(self, context):
        """
            Modifys content for ajax Partitials
            Detailed: It loads all defined files from partitials_list and render this templates

            :param context: context dict
            :return: context dict
        """
        partitial_ctx = dict()
        for selector, origin in self.get_partitial_list().items():
            partitial_ctx[selector] = self.get_partitial(origin, context)
        return partitial_ctx

    def is_ajax(self):
        if "is-ajax" in self.request.GET:
            return True
        return self.request.META.get('HTTP_X_REQUESTED_WITH') == 'XMLHttpRequest'

    def dispatch(self, *args, **kwargs):
        request = args[0]

        if not self.is_ajax():
            return super().dispatch(*args, **kwargs)
        else:
            if request.method.lower() in self.http_method_names:
                handler = getattr(self, f"ajax_{request.method.lower()}", self.http_method_not_allowed)
            else:
                handler = self.http_method_not_allowed
            return handler(*args, **kwargs)

    def generate_content(self, context):
        content_list = {}
        for key, item in context["partitial"].items():
            content_list[key] = item["content"]
        return content_list

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        print(ctx)
        ctx.update(self.get_direct_context_data(ctx))
        ctx.update({"partitial": self.get_partitial_context(ctx)})
        return ctx

    def get_direct_context_data(self, ctx):
        return {}

    def ajax_get(self, *args, **kwargs):
        ctx = self.get_context_data()
        return JsonResponse({
            "content": self.generate_content(ctx)
        })


class DeletePartitialAjaxMixin(PartitialAjaxMixin):

    def ajax_get(self, *args, **kwargs):
        self.object = self.get_object()
        ctx = self.get_context_data(object=self.object)
        return JsonResponse({
            "content": self.generate_content(ctx)
        })

    def ajax_post(self, *args, **kwargs):
        super().post(*args, **kwargs)
        return JsonResponse({
            "status": "ok"
        })


class CreatePartitialAjaxMixin(PartitialAjaxMixin):

    def ajax_get(self, *args, **kwargs):
        self.object = None
        return super().ajax_get(*args, **kwargs)

    def ajax_post(self, *args, **kwargs):
        super().post(*args, **kwargs)
        return JsonResponse({
            "status": "ok"
        })


class ListPartitialAjaxMixin(PartitialAjaxMixin):

    def ajax_get(self, *args, **kwargs):
        self.object_list = self.get_queryset()
        return super().ajax_get(*args, **kwargs)


class UpdatePartitialAjaxMixin(PartitialAjaxMixin):
    pass


class DetailPartitialAjaxMixin(PartitialAjaxMixin):

    def ajax_get(self, *args, **kwargs):
        self.object = self.get_object()
        ctx = self.get_context_data(object=self.object)
        return JsonResponse({
            "content": self.generate_content(ctx)
        })


from django import template
from partitialajax.contrib.html import split_selector
from django.utils.translation import pgettext_lazy as __
import base64
from partitialajax.exceptions import PartitialNotFound

register = template.Library()


def _encode_partitial_parameter(data):
    """
        Helper method; It converts a python string to Base64 (using utf8)

        :param data: string
        :return: base64 encoded string
    """
    return base64.b64encode(data.encode("utf-8")).decode()


def fixbool(data):
    bool_attrs = ("only-child-replace", "restrict-remote-configuration", "config-from-element", "direct-load")
    for key, item in data.items():
        if key in bool_attrs:
            if item:
                return "true"
            return "false"

@register.inclusion_tag('partitialajax/partitial_include.html', takes_context=True)
def direct_partitial(context, selector, **kwargs):
    """
        Embedds a Partitial without initiial loading, or updates

        :param context: (Automatic Argument) Render Context
        :param selector: String which partitial should be included. (Same as defined in Partitial List)
        :param kwargs: Possible kwargs: allowed_methods, reload, url
        :return context: new dict with rended context
    """

    data = __("selector at template not defined in view error message", "Sorry unable to get Element to display Data")

    if "partitial" not in context:
        raise PartitialNotFound("No Partitials defined")

    if selector in context["partitial"]:
        data = context["partitial"][selector]["content"]
    else:
        raise PartitialNotFound(f"No Partitial with selector '{selector}' was found")

    splited_selector = split_selector(selector)
    if splited_selector["element"] is None:
        splited_selector["element"] = "div"
    splited_selector["class_list"].add("ajaxpartitial-container")

    context["partitial"] = {
        "data": data,
        "selector": splited_selector,
        "prefix": "partitial",
        "specific": {
            "url": kwargs.get("url", ""),
            "only-child-replace": kwargs.get("onlyChildReplace", True),
            "interval": kwargs.get("interval", -1),
            "allowed-elements": kwargs.get("allowedElements", "all"),
            "text-event-callback": kwargs.get("textEventCallback", "console.info"),
            "restrict-remote-configuration": kwargs.get("restrictRemoteConfiguration", False),
            "config-from-element": kwargs.get("configFromElement", True),
            "direct-load": kwargs.get("directLoad", False)
        }
    }

    context["partitial"]["specific"] = fixbool(context["partitial"]["specific"])

    return context



from django.test import TestCase
from partitialajax.exceptions import PartitialNotFound
from partitialajax.templatetags.partitialajax import direct_partitial
from django.template import Context, Template
import re

class DirectPartitialTemplateTagTestCase(TestCase):

    def test_no_partitial_context(self):
        context = Context({})
        template_to_render = Template(
            "{% load partitialajax %}"
            "{% direct_partitial 'specialElement#myid' %}"
        )

        with self.assertRaises(PartitialNotFound) as ctx:
            rendered_template = template_to_render.render(context)

    def test_no_selector_context(self):
        context = Context({"partitial": {"invalidselector":{}}})
        template_to_render = Template(
            "{% load partitialajax %}"
            "{% direct_partitial 'specialElement#myid' %}"
        )
        with self.assertRaises(PartitialNotFound) as ctx:
            rendered_template = template_to_render.render(context)


    def test_with_selector_context(self):
        context = Context({"partitial": {"specialElement#myid":{
            "content": "hallotest"
        }}})
        template_to_render = Template(
            "{% load partitialajax %}"
            "{% direct_partitial 'specialElement#myid' %}"
        )
        html = template_to_render.render(context)

        self.assertEqual(re.sub(r'\s*', '', """<specialElementid="myid"class="ajaxpartitial-container"data-partitial-url=""data-partitial-only-child-replace="True"data-partitial-interval="-1"data-partitial-allowed-elements="all"data-partitial-text-event-callback="console.info"data-partitial-restrict-remote-configuration="False"data-partitial-config-from-element="True"data-partitial-direct-load="False"data-partitial-activate=true>hallotest</div>"""),
                         re.sub(r'\s*', '', html))


# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
import django
import sphinx_theme
from django.conf import settings

sys.path.insert(0, os.path.abspath('../../'))

# pass settings into configure
settings.configure(
    INSTALLED_APPS = [
        'django.contrib.admin',
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.messages',
        'django.contrib.staticfiles',
        'partitialajax',
    ]
)

# call django.setup to load installed apps and other stuff
django.setup()


# -- Project information -----------------------------------------------------

project = 'django-partitialajax'
copyright = '2019-2020, Stefan Eiermann'
author = 'Stefan Eiermann'


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.coverage',
    'sphinx_js',
    'sphinx-jsonschema'

]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'
# html_theme_path = [sphinx_theme.get_html_theme_path()]


# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
#html_static_path = ['_static']


js_source_path = os.path.abspath("../../partitialajax/src/js/")
print(js_source_path)

master_doc = 'index'
autosectionlabel_prefix_document = True
